
/*
 *	$Header: /home/glenn/src/labpc/RCS/counter.c,v 0.2 1995/10/24 16:32:20 glenn Exp glenn $
 *
 *	$Log: counter.c,v $
 *	Revision 0.2  1995/10/24 16:32:20  glenn
 *	Revision 0.2
 *
 * Revision 0.1  1994/11/28  04:41:15  glenn
 * Initial release - supports DACs and counter/timers
 *
 * Revision 1.2  1994/09/05  20:04:50  glenn
 * Fixed initialisation of Lab-PC+ card at start-up - may be more
 * reliable to detect now.
 *
 * Revision 1.1  1994/09/05  14:47:50  glenn
 * Initial revision
 *
 *
 */

#include <linux/config.h>

#ifdef MODULE
#include <linux/module.h>
#include <linux/version.h>
#else
#define MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif MODULE

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/config.h>

#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>

#include "counter.h"
#include "labpc.h"

static int write_counter( struct inode *inode, struct file *file,
		        const char *buffer, int count )
{
	Counter_t	*counter;

	dprintk(( "LabPC+: write_counter()\n" ));
		
	/* Must write 16 bit int to counter */
	if (count != sizeof( ushort ))
		return -(EINVAL);

	/* Which counter? */
	if ((counter = GETCOUNTER( MINOR(inode->i_rdev) )) == NULL)
		return -(EINVAL);

	Write8253( counter, get_fs_word( buffer ) );

	return sizeof( short );
}

static int read_counter( struct inode *inode, struct file *file,
			char *buffer, int count )
{
	Counter_t	*counter;
	
	dprintk(( "LabPC+: read_counter()\n" ));
		
	/* Must read 16 bit int to counter */
	if (count < sizeof( ushort ))
		return -(EINVAL);

	/* Which counter? */
	if ((counter = GETCOUNTER( MINOR(inode->i_rdev) )) == NULL)
		return -(EINVAL);

	put_fs_word( Read8253( counter ), (short *)buffer );

	return sizeof( short );
}

static int ioctl_counter( struct inode *inode, struct file *file,
			 unsigned int cmd, unsigned long arg )
{
	Counter_t	*counter;
	int		stat;

	dprintk(( "LabPC+: ioctl_counter()\n" ));
		
	if ((stat = labpc_ioctl_check( cmd, arg )) != 0)
		return stat;
		
	/* Which counter? */
	if ((counter = GETCOUNTER( MINOR(inode->i_rdev) )) == NULL)
		return -(EINVAL);

	switch (cmd) {
	case COUNTER_SETMODE:
		/*
		 * Set the mode for this counter. This will not affect
		 * currently running counts. The new mode will be implemented
		 * on the next "write" to the counter
		 */
		/* Check we have a valid mode */
		if (arg >= NUMBER_OF_MODES)
			return -(EINVAL);	

		return SetMode8253( counter, arg );
		break;
	default:
		return -(EINVAL);
	}

	return 0;			/* NoERR */
}

static int open_counter( struct inode *inode, struct file *file )
{
	int		minor;
	Counter_t	*counter;

	dprintk(( "LabPC+: open_counter()\n" ));
		
	/* Which counter? */
	minor = MINOR(inode->i_rdev);
	if ((counter = GETCOUNTER( minor )) == NULL)
		return -(EINVAL);

	return Open8253( counter );
}

static void release_counter( struct inode *inode, struct file *file )
{
	Counter_t *counter;

	dprintk(( "LabPC+: release_counter()\n" ));
		
	if ((counter = GETCOUNTER( MINOR(inode->i_rdev) )) == NULL)
		return;

	Close8253( counter );
	labpc_close( inode, file );
}

struct file_operations counter_fops = {
	NULL,				/* seek */
	read_counter,
	write_counter,
	NULL,				/* readdir */
	NULL,				/* select_counter */
	ioctl_counter,
	NULL,				/* mmap */
	open_counter,
	release_counter,
};

#define LSBMSB		(RL0 | RL1)

static struct semaphore countersemid = MUTEX;

int Write8253( Counter_t *counter, unsigned int data )
{
	unsigned char	c,	lo,	hi;

	/* Wait on our exclusion semaphore */
	down( &countersemid );

	/* Load the mode byte */
	c = 	(LSBMSB & 0x30) |
		((counter->num << 6) & 0xc0) |
		((counter->mode << 1) & 0x0e) |
		((counter->bcd) & 0x01);
	counter->status |= COUNTER_COUNTING;
	
	/* Write the configuration to the mode port */
	outb_p( c, counter->mode_port );

	/* Write value to the counter - LSB then MSB */
	lo = data & 0xff;
	outb_p( lo, counter->data_port );
	hi = (data >> 8) & 0xff;
	outb_p( hi, counter->data_port );

	/* Release the semaphore */
	up( &countersemid );

	return 0;
}

unsigned int Read8253( Counter_t *counter )
{
	int	lsb,	msb;
	
	/* Wait for our semphore lock */
	down( &countersemid );

	/* Load the mode port with a Counter Latch Command (RL0 = RL1 = 0) */
	outb_p( (counter->num << 6), counter->mode_port );

	/* Read value from the counter - LSB then MSB */
	lsb = inb_p( counter->data_port );
	msb = inb_p( counter->data_port );
	
	/* Release the semaphore */
	up( &countersemid );

	return (((msb << 8) & 0xff00) | (lsb & 0xff));
}

int SetLow8253( Counter_t *counter )
{
	unsigned char	c;
	
	/* Wait for our semphore lock */
	down( &countersemid );

	/* Load the mode byte */
	c = 	(LSBMSB & 0x30) |
		((counter->num << 6) & 0xc0) |
		((MODE_TERMINAL_COUNT << 1) & 0x0e) |
		((counter->bcd) & 0x01);

	outb_p( c, counter->mode_port );

	/* Release the semaphore */
	up( &countersemid );

	return 0;
}

int SetHigh8253( Counter_t *counter )
{
	unsigned char	c;
	
	/* Wait for our semphore lock */
	down( &countersemid );

	/* Load the mode byte */
	c = 	(LSBMSB & 0x30) |
		((counter->num << 6) & 0xc0) |
		((MODE_RATE_GENERATOR << 1) & 0x0e) |
		((counter->bcd) & 0x01);

	outb_p( c, counter->mode_port );

	/* Release the semaphore */
	up( &countersemid );

	return 0;
}

int SetMode8253( Counter_t *counter, int mode )
{
	/*	
	 * Set the mode for this counter. This will not affect
	 * currently running counts. The new mode will be implemented
	 * on the next "write" to the counter
	 */

	/* Wait for our semphore lock */
	down( &countersemid );

	counter->mode = mode;

	/* Release the semaphore */
	up( &countersemid );

	return 0;			/* NoERR */
}

int SetCountStyle( Counter_t *counter, int bcd )
{
	/* Wait for our semphore lock */
	down( &countersemid );

	counter->bcd = (bcd != 0);

	/* Release the semaphore */
	up( &countersemid );

	return 0;
}

int Open8253( Counter_t *counter )
{
	/* Initialise the counter structure */
	if (counter->first_time) {
		counter->first_time = 0;
		
		counter->status = 0;	/* Initialised			*/
		counter->mode = 2;	/* Set to mode 2 counting	*/
		counter->bcd = 0;	/* Binary mode counting		*/
		counter->open = 0;	/* We are open for business	*/
	}

	counter->open++;

	return 0;
}

int Close8253( Counter_t *counter )
{
	counter->status = 0;
	counter->open--;

	return 0;
}

int Init8253( Counter_t *counter, int minor, int num, int data_port, int mode_port )
{
	counter->first_time = 1;
	counter->data_port = data_port;
	counter->mode_port = mode_port;
	counter->num = num;
	counter->minor = minor;
	counter->mode = 2;
	counter->bcd = 0;
	counter->status = 0;
	counter->open = 0;

	return 0;
}
