#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "labpc.h"

#define MAXVALUES	5000


void Usage( void )
{
	fprintf( stderr, "Usage: adcread -dma on/off -scan on/off -clock count -gain 1/1.25/2/5/10/20/50/100 -unipolar -bipolar -channel number -timebase microseconds -trigger soft/hard/pre/auto -singleended -differential -buf adcbufsize -n #ofsamples -h device_file_name\n" );
}

int main( int argc, char **argv )
{
	short		value[MAXVALUES];
	char		*filename;
	int		scan = -1;
	int		gain = -1;
	int		channel = -1;
	int		trigger = -1;
	int		connection = -1;
	int		tbase = -1;
	int		polarity = -1;
	int		numsamples = 1;
	int		clock = -1;
	int		dma = -1;
	int		bufsize = -1;
	int		fd;
	int		i,	n;
	float		d;
	float		cal = 1.0;

	while (*++argv && **argv == '-') {
		if (!strcmp( *argv, "-scan" )) {
			if (!strcmp( *++argv, "on" )) {
				scan = ADC_SCAN_ON;
			} else if (!strcmp( *argv, "off" )) {
				scan = ADC_SCAN_OFF;
			} else {
				fprintf( stderr, "Invalid scan option %s\n", *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-dma" )) {
			if (!strcmp( *++argv, "on" )) {
				dma = 1;
			} else if (!strcmp( *argv, "off" )) {
				dma = 0;
			} else {
				fprintf( stderr, "Invalid dma option %s\n", *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-gain" )) {
			if (!strcmp( *++argv, "1" )) {
				gain = ADC_GAIN_1;
				cal = 1.0;
			} else if (!strcmp( *argv, "1.25" )) {
				gain = ADC_GAIN_1p25;
				cal = 1.25;
			} else if (!strcmp( *argv, "2" )) {
				gain = ADC_GAIN_2;
				cal = 2.0;
			} else if (!strcmp( *argv, "5" )) {
				gain = ADC_GAIN_5;
				cal = 5.0;
			} else if (!strcmp( *argv, "10" )) {
				gain = ADC_GAIN_10;
				cal = 10.0;
			} else if (!strcmp( *argv, "20" )) {
				gain = ADC_GAIN_20;
				cal = 20.0;
			} else if (!strcmp( *argv, "50" )) {
				gain = ADC_GAIN_50;
				cal = 50.0;
			} else if (!strcmp( *argv, "100" )) {
				gain = ADC_GAIN_100;
				cal = 100.0;
			} else {
				fprintf( stderr, "Invalid gain value \"%s\"\n",
					 *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-unipolar" )) {
			polarity = ADC_UNIPOLAR;
		} else if (!strcmp( *argv, "-bipolar" )) {
			polarity = ADC_BIPOLAR;
		} else if (!strcmp( *argv, "-channel" )) {
			if (sscanf( *++argv, "%i", &channel ) != 1) {
				fprintf( stderr, "Invalid channel \"%s\"\n",
					 *argv );
				Usage();
				exit( 2 );
			}
			if (channel < 0 || channel >= ADC_NUM_CHANNELS) {
				fprintf( stderr, "Channel number %i out of range - [0-%d]\n",
					 channel, ADC_NUM_CHANNELS - 1 );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-timebase" )) {
			if (sscanf( *++argv, "%g", &d ) != 1) {
				fprintf( stderr, "Invalid timebase: %s\n", *argv );
				Usage();
				exit( 2 );
			}
			tbase = d * 2.0 + 0.5;
		} else if (!strcmp( *argv, "-clock" )) {
			if (sscanf( *++argv, "%d", &clock ) != 1) {
				fprintf( stderr, "Invalid clock: %s\n", *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-trigger" )) {
			if (!strcmp( *++argv, "soft" )) {
				trigger = ADC_SOFTTRIGGER;
			} else if (!strcmp( *argv, "hard" )) {
				trigger = ADC_HARDTRIGGER;
			} else if (!strcmp( *argv, "pre" )) {
				trigger = ADC_PRETRIGGER;
			} else if (!strcmp( *argv, "auto" )) {
				trigger = ADC_AUTOTRIGGER;
			} else {
				fprintf( stderr, "Invalid trigger \"%s\"\n",
					 *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-singleended" )) {
			connection = ADC_SINGLEENDED;
		} else if (!strcmp( *argv, "-differential" )) {
			connection = ADC_DIFFERENTIAL;
		} else if (!strcmp( *argv, "-buf" )) {
			if (sscanf( *++argv, "%i", &bufsize ) != 1) {
				fprintf( stderr, "Invalid buffer size \"%s\"\n",
					 *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-n" )) {
			if (sscanf( *++argv, "%i", &numsamples ) != 1) {
				fprintf( stderr, "Invalid sample option \"%s\"\n",
					 *argv );
				Usage();
				exit( 2 );
			}
		} else if (!strcmp( *argv, "-h" )) {
			Usage();
			exit( 1 );
		} else {
			fprintf( stderr, "Unknown option: %s\n", *argv );
			Usage();
			exit( 2 );
		}
	}

	if ((filename = *argv) == NULL) {
		Usage();
		exit( 1 );
	}

	if ((fd = open( filename, O_RDWR )) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (scan >= 0 && ioctl( fd, ADC_SET_SCAN, scan ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (dma >= 0 && ioctl( fd, ADC_DMAEN, dma ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (gain >= 0 && ioctl( fd, ADC_SET_GAIN, gain ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (polarity >= 0 && ioctl( fd, ADC_SET_POLARITY, polarity ) < 0) {
		perror( filename );
		exit( 2 );
	}
		
	if (channel >= 0 && ioctl( fd, ADC_SET_CHANNEL, channel ) < 0) {
		perror( filename );
		exit( 2 );
	}
		
	if (tbase >= 0 && ioctl( fd, ADC_SET_TIMEBASE, tbase ) < 0) {
		perror( filename );
		exit( 2 );
	}
		
	if (trigger >= 0 && ioctl( fd, ADC_SET_TRIGGER_MODE, trigger ) < 0) {
		perror( filename );
		exit( 2 );
	}
		
	if (clock >= 0 && ioctl( fd, ADC_SET_CLOCK, clock ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (connection >= 0 && ioctl( fd, ADC_SE_D, connection ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (bufsize >= 0 && ioctl( fd, ADC_SETBUFSIZE, bufsize ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (ioctl( fd, ADC_GET_SCAN, &scan ) < 0) {
		perror( filename );
		exit( 2 );
	}
		
	if (ioctl( fd, ADC_GET_CHANNEL, &channel ) < 0) {
		perror( filename );
		exit( 2 );
	}

	if (numsamples > MAXVALUES)
		numsamples = MAXVALUES;

	if (numsamples) {
		if ((n = read( fd, value, numsamples * sizeof( short ) )) < 0) {
			perror( filename );
			exit( 2 );
		}
		n /= sizeof( short );
		for (i = 0; i < n; i++) {
			printf( "%g\n", value[i] * (10.0 / cal / 4096) );
		}
	}
	
	close( fd );

	return 0;
}

	
	
	
	
